@@ -1,7 +1,7 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
 # License: GPLv3 Copyright: 2016, Kovid Goyal <kovid at kovidgoyal.net>
-from __future__ import absolute_import, division, print_function, unicode_literals
+
 
 import os, sys, subprocess, binascii, json
 
@@ -26,7 +26,7 @@ def get_exe():
     return 'python3' if sys.version_info.major == 2 else sys.executable
 
 
-def get_cmd(exe, bypy, which, bitness, sign_installers, notarize=True):
+def get_cmd(exe, bypy, which, bitness, sign_installers, notarize=True, compression_level='9'):
     cmd = [exe, bypy, which]
     if bitness and bitness == '32':
         cmd.append(bitness)
@@ -37,6 +37,7 @@ def get_cmd(exe, bypy, which, bitness, sign_installers, notarize=True):
         cmd.append('--sign-installers')
     if notarize:
         cmd.append('--notarize')
+    cmd.append('--compression-level=' + compression_level)
     return cmd
 
 
@@ -67,10 +68,10 @@ def build_only(which, bitness, spec, shutdown=False):
     return dist
 
 
-def build_single(which='windows', bitness='64', shutdown=True, sign_installers=True, notarize=True):
+def build_single(which='windows', bitness='64', shutdown=True, sign_installers=True, notarize=True, compression_level='9'):
     base, bypy = get_paths()
     exe = get_exe()
-    cmd = get_cmd(exe, bypy, which, bitness, sign_installers, notarize)
+    cmd = get_cmd(exe, bypy, which, bitness, sign_installers, notarize, compression_level=compression_level)
     ret = subprocess.Popen(cmd).wait()
     if ret != 0:
         raise SystemExit(ret)
@@ -123,9 +124,19 @@ class BuildInstaller(Command):
             action='store_true',
             help='Do not notarize the installers'
         )
+        parser.add_option(
+            '--compression-level',
+            default='9',
+            choices=list('123456789'),
+            help='Do not notarize the installers'
+        )
 
     def run(self, opts):
-        build_single(self.OS, self.BITNESS, not opts.dont_shutdown, not opts.dont_sign, not opts.dont_notarize)
+        build_single(
+            self.OS, self.BITNESS, not opts.dont_shutdown,
+            not opts.dont_sign, not opts.dont_notarize,
+            compression_level=opts.compression_level
+        )
 
 
 class BuildInstallers(BuildInstaller):
@@ -180,11 +191,16 @@ class BuildDep(Command):
     description = (
         'Build a calibre dependency. For e.g. build_dep windows expat.'
         ' Without arguments builds all deps for specified platform. Use windows 32 for 32bit.'
+        ' Use build_dep all somedep to build a dep for all platforms.'
     )
 
     def run(self, opts):
         args = opts.cli_args
-        build_dep(args)
+        if args and args[0] == 'all':
+            for x in ('linux', 'linux 32', 'macos', 'windows', 'windows 32'):
+                build_dep(x.split() + list(args)[1:])
+        else:
+            build_dep(args)
 
 
 class ExportPackages(Command):
@@ -194,7 +210,7 @@ class ExportPackages(Command):
     def run(self, opts):
         base, bypy = get_paths()
         exe = get_exe()
-        cmd = [exe, bypy, 'export'] + list(opts.cli_args) + ['download.calibre-ebook.com:/srv/download/ci/calibre']
+        cmd = [exe, bypy, 'export'] + list(opts.cli_args) + ['download.calibre-ebook.com:/srv/download/ci/calibre3']
         ret = subprocess.Popen(cmd).wait()
         if ret != 0:
             raise SystemExit(ret)
