@@ -18,12 +18,13 @@ is_ci = os.environ.get('CI', '').lower() == 'true'
 
 
 def qt_sources():
-    qtdir = '/usr/src/qt5'
+    qtdir = os.environ.get('QT_SRC', '/usr/src/qt5/qtbase')
     j = partial(os.path.join, qtdir)
     return list(map(j, [
-            'qtbase/src/gui/kernel/qplatformtheme.cpp',
-            'qtbase/src/widgets/dialogs/qcolordialog.cpp',
-            'qtbase/src/widgets/dialogs/qfontdialog.cpp',
+            'src/gui/kernel/qplatformtheme.cpp',
+            'src/widgets/dialogs/qcolordialog.cpp',
+            'src/widgets/dialogs/qfontdialog.cpp',
+            'src/widgets/widgets/qscrollbar.cpp',
     ]))
 
 
@@ -91,6 +92,36 @@ class POT(Command):  # {{{
 
         return '\n'.join(ans)
 
+    def get_iso639_strings(self):
+        self.info('Generating translation template for iso639')
+        src = self.j(self.d(self.SRC), 'setup', 'iso_639-3.json')
+        if not os.path.exists(src):
+            raise Exception(src + ' does not exist')
+        with open(src, 'rb') as f:
+            root = json.load(f)
+        entries = root['639-3']
+        ans = []
+
+        def name_getter(x):
+            return x.get('inverted_name') or x.get('name')
+
+        for x in sorted(entries, key=lambda x:name_getter(x).lower()):
+            name = name_getter(x)
+            if name:
+                ans.append(u'msgid "{}"'.format(name))
+                ans.append('msgstr ""')
+                ans.append('')
+        pot = self.pot_header() + '\n\n' + '\n'.join(ans)
+        dest = self.j(self.TRANSLATIONS, 'iso_639', 'iso_639_3.pot')
+        try:
+            os.makedirs(os.path.dirname(dest))
+        except OSError:
+            pass
+        with open(dest, 'wb') as f:
+            f.write(pot.encode('utf-8'))
+        self.upload_pot(resource='iso639')
+        self.git(['add', dest])
+
     def get_content_server_strings(self):
         self.info('Generating translation template for content_server')
         from calibre import walk
@@ -106,7 +137,7 @@ class POT(Command):  # {{{
     def get_user_manual_docs(self):
         self.info('Generating translation templates for user_manual')
         base = tempfile.mkdtemp()
-        subprocess.check_call(['calibre-debug', self.j(self.d(self.SRC), 'manual', 'build.py'), 'gettext', base])
+        subprocess.check_call([sys.executable, self.j(self.d(self.SRC), 'manual', 'build.py'), 'gettext', base])
         tbase = self.j(self.TRANSLATIONS, 'manual')
         for x in os.listdir(base):
             if not x.endswith('.pot'):
@@ -181,8 +212,11 @@ class POT(Command):  # {{{
                 time=time.strftime('%Y-%m-%d %H:%M+%Z'))
 
     def run(self, opts):
-        require_git_master()
-        self.get_website_strings()
+        if not is_ci:
+            require_git_master()
+        self.get_iso639_strings()
+        if not is_ci:
+            self.get_website_strings()
         self.get_content_server_strings()
         self.get_user_manual_docs()
         files = self.source_files()
@@ -190,7 +224,7 @@ class POT(Command):  # {{{
         pot_header = self.pot_header()
 
         with tempfile.NamedTemporaryFile() as fl:
-            fl.write('\n'.join(files))
+            fl.write('\n'.join(files).encode('utf-8'))
             fl.flush()
             out = tempfile.NamedTemporaryFile(suffix='.pot', delete=False)
             out.close()
@@ -207,7 +241,7 @@ class POT(Command):  # {{{
                 ] + qt_inputs)
 
             with open(out.name, 'rb') as f:
-                src = f.read()
+                src = f.read().decode('utf-8')
             os.remove(out.name)
             src = pot_header + '\n' + src
             src += '\n\n' + self.get_tweaks_docs()
@@ -220,12 +254,12 @@ class POT(Command):  # {{{
             src = re.sub(r'#, python-brace-format\s+msgid ""\s+.*<code>{0:</code>',
                    lambda m: m.group().replace('python-brace', 'no-python-brace'), src)
             with open(pot, 'wb') as f:
-                f.write(src)
+                f.write(src.encode('utf-8'))
             self.info('Translations template:', os.path.abspath(pot))
             self.upload_pot(resource='main')
             self.git(['add', os.path.abspath(pot)])
 
-        if self.git('diff-index --cached --quiet --ignore-submodules HEAD --', use_call=True) != 0:
+        if not is_ci and self.git('diff-index --cached --quiet --ignore-submodules HEAD --', use_call=True) != 0:
             self.git(['commit', '-m', 'Updated translation templates'])
             self.git('push')
 
@@ -528,7 +562,7 @@ class Translations(POT):  # {{{
     def _compile_website_translations(self, name='website', threshold=50):
         from calibre.utils.zipfile import ZipFile, ZipInfo, ZIP_STORED
         from calibre.ptempfile import TemporaryDirectory
-        from calibre.utils.localization import get_iso639_translator, get_language, get_iso_language
+        from calibre.utils.localization import get_language, translator_for_lang
         self.info('Compiling', name, 'translations...')
         srcbase = self.j(self.d(self.SRC), 'translations', name)
         if not os.path.exists(srcbase):
@@ -569,11 +603,9 @@ class Translations(POT):  # {{{
 
             lang_names = {}
             for l in dl:
-                if l == 'en':
-                    t = get_language
-                else:
-                    t = get_iso639_translator(l).gettext
-                    t = partial(get_iso_language, t)
+                translator = translator_for_lang(l)['translator']
+                t = translator.gettext
+                t = partial(get_language, gettext_func=t)
                 lang_names[l] = {x: t(x) for x in dl}
             zi = ZipInfo('lang-names.json')
             zi.compress_type = ZIP_STORED
@@ -660,7 +692,7 @@ class GetTranslations(Translations):  # {{{
         if opts.check_for_errors:
             self.check_all()
             return
-        self.tx('pull -a')
+        self.tx('pull -a --parallel --no-interactive')
         if not self.is_modified:
             self.info('No translations were updated')
             return
@@ -773,7 +805,7 @@ class GetTranslations(Translations):  # {{{
             for f in errfiles:
                 with open(f, 'r+b') as f:
                     raw = f.read()
-                    raw = re.sub(r'# \(pofilter\).*', '', raw)
+                    raw = re.sub(rb'# \(pofilter\).*', b'', raw)
                     f.seek(0)
                     f.truncate()
                     f.write(raw)
@@ -827,7 +859,7 @@ class ISO639(Command):  # {{{
                 threeb = unicode_type(threeb)
             if threeb is None:
                 continue
-            name = x.get('name')
+            name = x.get('inverted_name') or x.get('name')
             if name:
                 name = unicode_type(name)
             if not name or name[0] in '!~=/\'"':
