@@ -503,6 +503,18 @@ def serialize_user_metadata(metadata_elem, all_user_metadata, tail='\n'+(' '*8))
         metadata_elem.append(meta)
 
 
+def serialize_annotations(metadata_elem, annotations, tail='\n'+(' '*8)):
+    for item in annotations:
+        data = json.dumps(item, ensure_ascii=False)
+        if isinstance(data, bytes):
+            data = data.decode('utf-8')
+        meta = metadata_elem.makeelement('meta')
+        meta.set('name', 'calibre:annotation')
+        meta.set('content', data)
+        meta.tail = tail
+        metadata_elem.append(meta)
+
+
 def dump_dict(cats):
     if not cats:
         cats = {}
@@ -647,6 +659,13 @@ class OPF(object):  # {{{
 
         return ans
 
+    def read_annotations(self):
+        for elem in self.root.xpath('//*[name() = "meta" and @name = "calibre:annotation" and @content]'):
+            try:
+                yield json.loads(elem.get('content'))
+            except Exception:
+                pass
+
     def write_user_metadata(self):
         elems = self.root.xpath('//*[name() = "meta" and starts-with(@name,'
                 '"calibre:user_metadata:") and @content]')
@@ -1294,7 +1313,9 @@ class OPF(object):  # {{{
                      'isbn', 'tags', 'category', 'comments', 'book_producer',
                      'pubdate', 'user_categories', 'author_link_map'):
             val = getattr(mi, attr, None)
-            is_null = val is None or val in ((), [], (None, None), {}) or (attr == 'rating' and val < 0.1)
+            if attr == 'rating' and val:
+                val = float(val)
+            is_null = val is None or val in ((), [], (None, None), {}) or (attr == 'rating' and (not val or val < 0.1))
             if is_null:
                 if apply_null and attr in {'series', 'tags', 'isbn', 'comments', 'publisher', 'rating'}:
                     setattr(self, attr, ([] if attr == 'tags' else None))
@@ -1306,6 +1327,7 @@ class OPF(object):  # {{{
         if apply_null or langs:
             self.languages = langs or []
         temp = self.to_book_metadata()
+        temp.remove_stale_user_metadata(mi)
         temp.smart_update(mi, replace_metadata=replace_metadata)
         if not replace_metadata and callable(getattr(temp, 'custom_field_keys', None)):
             # We have to replace non-null fields regardless of the value of
@@ -1664,6 +1686,9 @@ def metadata_to_opf(mi, as_string=True, default_lang=None):
         meta('user_categories', dump_dict(mi.user_categories))
 
     serialize_user_metadata(metadata, mi.get_all_user_metadata(False))
+    all_annotations = getattr(mi, 'all_annotations', None)
+    if all_annotations:
+        serialize_annotations(metadata, all_annotations)
 
     metadata[-1].tail = '\n' +(' '*4)
 
