@@ -1,15 +1,15 @@
 # -*- coding: utf-8 -*-
 
+
 __license__ = 'GPL 3'
 __copyright__ = '2009, John Schember <john@nachtimwald.com>'
 __docformat__ = 'restructuredtext en'
 
 import shutil
 
-from PyQt5.Qt import QModelIndex, QDialog
+from PyQt5.Qt import QModelIndex, QDialog, QApplication
 
-from calibre.gui2.convert.single import (Config, sort_formats_by_preference,
-    GroupModel, gprefs, get_output_formats)
+from calibre.gui2.convert.single import Config, GroupModel, gprefs
 from calibre.gui2.convert.look_and_feel import LookAndFeelWidget
 from calibre.gui2.convert.heuristics import HeuristicsWidget
 from calibre.gui2.convert.search_and_replace import SearchAndReplaceWidget
@@ -18,16 +18,23 @@ from calibre.gui2.convert.structure_detection import StructureDetectionWidget
 from calibre.gui2.convert.toc import TOCWidget
 from calibre.gui2.convert import GuiRecommendations
 from calibre.ebooks.conversion.plumber import Plumber
+from calibre.ebooks.conversion.config import sort_formats_by_preference, get_output_formats
 from calibre.utils.config import prefs
 from calibre.utils.logging import Log
+from polyglot.builtins import unicode_type, native_string_type
 
 
 class BulkConfig(Config):
 
     def __init__(self, parent, db, preferred_output_format=None,
-            has_saved_settings=True):
+            has_saved_settings=True, book_ids=()):
         QDialog.__init__(self, parent)
-        self.setupUi(self)
+        self.widgets = []
+        self.setupUi()
+        try:
+            self.num_of_books = len(book_ids)
+        except Exception:
+            self.num_of_books = 1
 
         self.setup_output_formats(db, preferred_output_format)
         self.db = db
@@ -43,7 +50,7 @@ class BulkConfig(Config):
             'values saved in a previous conversion (if they exist) instead '
             'of using the defaults specified in the Preferences'))
 
-        self.output_formats.currentIndexChanged[str].connect(self.setup_pipeline)
+        self.output_formats.currentIndexChanged[native_string_type].connect(self.setup_pipeline)
         self.groups.setSpacing(5)
         self.groups.activated[(QModelIndex)].connect(self.show_pane)
         self.groups.clicked[(QModelIndex)].connect(self.show_pane)
@@ -60,7 +67,7 @@ class BulkConfig(Config):
 
         geom = gprefs.get('convert_bulk_dialog_geom', None)
         if geom:
-            self.restoreGeometry(geom)
+            QApplication.instance().safe_restore_geometry(self, geom)
         else:
             self.resize(self.sizeHint())
 
@@ -76,10 +83,12 @@ class BulkConfig(Config):
         self.plumber.merge_plugin_recs(self.plumber.output_plugin)
 
         def widget_factory(cls):
-            return cls(self.stack, self.plumber.get_option_by_name,
+            return cls(self, self.plumber.get_option_by_name,
                 self.plumber.get_option_help, self.db)
 
-        self.setWindowTitle(_('Bulk convert'))
+        self.setWindowTitle(
+            ngettext(_('Bulk convert one book'), _('Bulk convert {} books'), self.num_of_books).format(self.num_of_books)
+        )
         lf = widget_factory(LookAndFeelWidget)
         hw = widget_factory(HeuristicsWidget)
         sr = widget_factory(SearchAndReplaceWidget)
@@ -89,28 +98,23 @@ class BulkConfig(Config):
         toc.manually_fine_tune_toc.hide()
 
         output_widget = self.plumber.output_plugin.gui_configuration_widget(
-                self.stack, self.plumber.get_option_by_name,
+                self, self.plumber.get_option_by_name,
                 self.plumber.get_option_help, self.db)
 
-        while True:
-            c = self.stack.currentWidget()
-            if not c:
-                break
-            self.stack.removeWidget(c)
-
-        widgets = [lf, hw, ps, sd, toc, sr]
+        self.break_cycles()
+        widgets = self.widgets = [lf, hw, ps, sd, toc, sr]
         if output_widget is not None:
             widgets.append(output_widget)
         for w in widgets:
-            self.stack.addWidget(w)
             w.set_help_signal.connect(self.help.setPlainText)
+            w.setVisible(False)
 
         self._groups_model = GroupModel(widgets)
         self.groups.setModel(self._groups_model)
 
         idx = oidx if -1 < oidx < self._groups_model.rowCount() else 0
         self.groups.setCurrentIndex(self._groups_model.index(idx))
-        self.stack.setCurrentIndex(idx)
+        self.show_pane(idx)
         try:
             shutil.rmtree(self.plumber.archive_input_tdir, ignore_errors=True)
         except:
@@ -124,8 +128,7 @@ class BulkConfig(Config):
             preferred_output_format and preferred_output_format \
             in output_formats else sort_formats_by_preference(output_formats,
                     [prefs['output_format']])[0]
-        self.output_formats.addItems(list(map(unicode, [x.upper() for x in
-            output_formats])))
+        self.output_formats.addItems((unicode_type(x.upper()) for x in output_formats))
         self.output_formats.setCurrentIndex(output_formats.index(preferred_output_format))
 
     def accept(self):
