@@ -37,6 +37,8 @@ class ImageView(QDialog):
 
     def __init__(self, parent, current_img, current_url, geom_name='viewer_image_popup_geometry'):
         QDialog.__init__(self)
+        self.setWindowFlag(Qt.WindowMinimizeButtonHint)
+        self.setWindowFlag(Qt.WindowMaximizeButtonHint)
         dw = QApplication.instance().desktop()
         self.avail_geom = dw.availableGeometry(parent if parent is not None else self)
         self.current_img = current_img
@@ -82,6 +84,9 @@ class ImageView(QDialog):
         h.addWidget(i), h.addStretch(), h.addWidget(bb)
         if self.fit_image.isChecked():
             self.set_to_viewport_size()
+        geom = gprefs.get(self.geom_name)
+        if geom is not None:
+            self.restoreGeometry(geom)
 
     def set_to_viewport_size(self):
         page_size = self.scrollarea.size()
@@ -98,15 +103,24 @@ class ImageView(QDialog):
         if self.fit_image.isChecked():
             self.set_to_viewport_size()
 
+    def factor_from_fit(self):
+        scaled_height = self.label.size().height()
+        actual_height = self.current_img.size().height()
+        return scaled_height / actual_height
+
     def zoom_in(self):
         if self.fit_image.isChecked():
+            factor = self.factor_from_fit()
             self.fit_image.setChecked(False)
+            self.factor = factor
         self.factor *= 1.25
         self.adjust_image(1.25)
 
     def zoom_out(self):
         if self.fit_image.isChecked():
+            factor = self.factor_from_fit()
             self.fit_image.setChecked(False)
+            self.factor = factor
         self.factor *= 0.8
         self.adjust_image(0.8)
 
