@@ -1,35 +1,38 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import itertools, operator, os, math
-from types import MethodType
-from threading import Event, Thread
-from Queue import LifoQueue
-from functools import wraps, partial
-from textwrap import wrap
-
+import itertools
+import math
+import operator
+import os
+from functools import wraps
 from PyQt5.Qt import (
-    QListView, QSize, QStyledItemDelegate, QModelIndex, Qt, QImage, pyqtSignal,
-    QTimer, QPalette, QColor, QItemSelection, QPixmap, QApplication,
-    QMimeData, QUrl, QDrag, QPoint, QPainter, QRect, pyqtProperty, QEvent,
-    QPropertyAnimation, QEasingCurve, pyqtSlot, QHelpEvent, QAbstractItemView,
-    QStyleOptionViewItem, QToolTip, QByteArray, QBuffer, QBrush, qRed, qGreen,
-    qBlue, QItemSelectionModel, QIcon, QFont)
+    QAbstractItemView, QApplication, QBuffer, QByteArray, QColor, QDrag,
+    QEasingCurve, QEvent, QFont, QHelpEvent, QIcon, QImage, QItemSelection,
+    QItemSelectionModel, QListView, QMimeData, QModelIndex, QPainter, QPixmap,
+    QPoint, QPropertyAnimation, QRect, QSize, QStyledItemDelegate,
+    QStyleOptionViewItem, Qt, QTableView, QTimer, QToolTip, QTreeView, QUrl,
+    pyqtProperty, pyqtSignal, pyqtSlot, qBlue, qGreen, qRed
+)
+from textwrap import wrap
+from threading import Event, Thread
 
-from calibre import fit_image, prints, prepare_string_for_xml, human_readable
+from calibre import fit_image, human_readable, prepare_string_for_xml, prints
 from calibre.constants import DEBUG, config_dir, islinux
-from calibre.gui2.pin_columns import PinContainer
 from calibre.ebooks.metadata import fmt_sidx, rating_to_stars
-from calibre.utils import join_with_timeout
-from calibre.gui2 import gprefs, config, rating_font, empty_index
+from calibre.gui2 import config, empty_index, gprefs, rating_font
+from calibre.gui2.dnd import path_from_qurl
 from calibre.gui2.gestures import GestureManager
 from calibre.gui2.library.caches import CoverCache, ThumbnailCache
+from calibre.gui2.pin_columns import PinContainer
+from calibre.utils import join_with_timeout
 from calibre.utils.config import prefs, tweaks
+from polyglot.builtins import itervalues, range, unicode_type
+from polyglot.queue import LifoQueue
 
 CM_TO_INCH = 0.393701
 CACHE_FORMAT = 'PPM'
@@ -77,7 +80,7 @@ def image_to_data(image):  # {{{
     buf.open(QBuffer.WriteOnly)
     if not image.save(buf, CACHE_FORMAT):
         raise EncodeError('Failed to encode thumbnail')
-    ret = bytes(ba.data())
+    ret = ba.data()
     buf.close()
     return ret
 # }}}
@@ -85,6 +88,13 @@ def image_to_data(image):  # {{{
 # Drag 'n Drop {{{
 
 
+def qt_item_view_base_class(self):
+    for q in (QTableView, QListView, QTreeView):
+        if isinstance(self, q):
+            return q
+    return QAbstractItemView
+
+
 def dragMoveEvent(self, event):
     event.acceptProposedAction()
 
@@ -95,14 +105,14 @@ def event_has_mods(self, event=None):
     return mods & Qt.ControlModifier or mods & Qt.ShiftModifier
 
 
-def mousePressEvent(base_class, self, event):
+def mousePressEvent(self, event):
     ep = event.pos()
     if self.indexAt(ep) in self.selectionModel().selectedIndexes() and \
             event.button() == Qt.LeftButton and not self.event_has_mods():
         self.drag_start_pos = ep
     if hasattr(self, 'handle_mouse_press_event'):
         return self.handle_mouse_press_event(event)
-    return base_class.mousePressEvent(self, event)
+    return qt_item_view_base_class(self).mousePressEvent(self, event)
 
 
 def drag_icon(self, cover, multiple):
@@ -178,11 +188,11 @@ def drag_data(self):
     return drag
 
 
-def mouseMoveEvent(base_class, self, event):
+def mouseMoveEvent(self, event):
     if not self.drag_allowed:
         return
     if self.drag_start_pos is None:
-        return base_class.mouseMoveEvent(self, event)
+        return qt_item_view_base_class(self).mouseMoveEvent(self, event)
 
     if self.event_has_mods():
         self.drag_start_pos = None
@@ -240,24 +250,20 @@ def paths_from_event(self, event):
     and represent files with extensions.
     '''
     md = event.mimeData()
-    if md.hasFormat('text/uri-list') and not \
-            md.hasFormat('application/calibre+from_library'):
-        urls = [unicode(u.toLocalFile()) for u in md.urls()]
-        return [u for u in urls if os.path.splitext(u)[1] and os.path.exists(u)]
+    if md.hasFormat('text/uri-list') and not md.hasFormat('application/calibre+from_library'):
+        urls = map(path_from_qurl, md.urls())
+        return [u for u in urls if u and os.path.splitext(u)[1] and os.path.exists(u)]
 
 
 def setup_dnd_interface(cls_or_self):
     if isinstance(cls_or_self, type):
         cls = cls_or_self
-        base_class = cls.__bases__[0]
         fmap = globals()
         for x in (
             'dragMoveEvent', 'event_has_mods', 'mousePressEvent', 'mouseMoveEvent',
             'drag_data', 'drag_icon', 'dragEnterEvent', 'dropEvent', 'paths_from_event'):
             func = fmap[x]
-            if x in {'mouseMoveEvent', 'mousePressEvent'}:
-                func = partial(func, base_class)
-            setattr(cls, x, MethodType(func, None, cls))
+            setattr(cls, x, func)
         return cls
     else:
         self = cls_or_self
@@ -326,7 +332,7 @@ class AlternateViews(object):
             view.setFocus(Qt.OtherFocusReason)
 
     def set_database(self, db, stage=0):
-        for view in self.views.itervalues():
+        for view in itervalues(self.views):
             if view is not self.main_view:
                 view.set_database(db, stage=stage)
 
@@ -355,7 +361,7 @@ class AlternateViews(object):
         self.current_view.select_rows(rows)
 
     def set_context_menu(self, menu):
-        for view in self.views.itervalues():
+        for view in itervalues(self.views):
             if view is not self.main_view:
                 view.set_context_menu(menu)
 
@@ -463,7 +469,7 @@ class CoverDelegate(QStyledItemDelegate):
                 if fm and fm['datatype'] == 'rating':
                     ans = rating_to_stars(val, fm['display'].get('allow_half_stars', False))
                     is_stars = True
-            return ('' if ans is None else unicode(ans)), is_stars
+            return ('' if ans is None else unicode_type(ans)), is_stars
         except Exception:
             if DEBUG:
                 import traceback
@@ -543,6 +549,10 @@ class CoverDelegate(QStyledItemDelegate):
             if self.emblem_size > 0:
                 self.paint_emblems(painter, rect, emblems)
             orect = QRect(rect)
+            trect = QRect(rect)
+            if self.title_height != 0:
+                rect.setBottom(rect.bottom() - self.title_height)
+                trect.setTop(trect.bottom() - self.title_height + 5)
             if cdata is None or cdata is False:
                 title = db.field_for('title', book_id, default_value='')
                 authors = ' & '.join(db.field_for('authors', book_id, default_value=()))
@@ -550,30 +560,20 @@ class CoverDelegate(QStyledItemDelegate):
                 painter.drawText(rect, Qt.AlignCenter|Qt.TextWordWrap, '%s\n\n%s' % (title, authors))
                 if cdata is False:
                     self.render_queue.put(book_id)
-            else:
                 if self.title_height != 0:
-                    trect = QRect(rect)
-                    rect.setBottom(rect.bottom() - self.title_height)
+                    self.paint_title(painter, trect, db, book_id)
+            else:
                 if self.animating is not None and self.animating.row() == index.row():
                     cdata = cdata.scaled(cdata.size() * self._animated_size)
                 dpr = cdata.devicePixelRatio()
                 cw, ch = int(cdata.width() / dpr), int(cdata.height() / dpr)
                 dx = max(0, int((rect.width() - cw)/2.0))
-                dy = max(0, rect.height() - ch)
+                dy = max(0, int((rect.height() - ch)/2.0))
                 right_adjust = dx
-                rect.adjust(dx, dy, -dx, 0)
+                rect.adjust(dx, dy, -dx, -dy)
                 painter.drawPixmap(rect, cdata)
                 if self.title_height != 0:
-                    rect = trect
-                    rect.setTop(rect.bottom() - self.title_height + 5)
-                    painter.setRenderHint(QPainter.TextAntialiasing, True)
-                    title, is_stars = self.render_field(db, book_id)
-                    if is_stars:
-                        painter.setFont(self.rating_font)
-                    metrics = painter.fontMetrics()
-                    painter.setPen(self.highlight_color)
-                    painter.drawText(rect, Qt.AlignCenter|Qt.TextSingleLine,
-                                     metrics.elidedText(title, Qt.ElideRight, rect.width()))
+                    self.paint_title(painter, trect, db, book_id)
             if self.emblem_size > 0:
                 return  # We dont draw embossed emblems as the ondevice/marked emblems are drawn in the gutter
             if marked:
@@ -592,6 +592,16 @@ class CoverDelegate(QStyledItemDelegate):
         finally:
             painter.restore()
 
+    def paint_title(self, painter, rect, db, book_id):
+        painter.setRenderHint(QPainter.TextAntialiasing, True)
+        title, is_stars = self.render_field(db, book_id)
+        if is_stars:
+            painter.setFont(self.rating_font)
+        metrics = painter.fontMetrics()
+        painter.setPen(self.highlight_color)
+        painter.drawText(rect, Qt.AlignCenter|Qt.TextSingleLine,
+                            metrics.elidedText(title, Qt.ElideRight, rect.width()))
+
     def paint_emblems(self, painter, rect, emblems):
         gutter = self.emblem_size + self.MARGIN
         grect = QRect(rect)
@@ -739,8 +749,8 @@ class GridView(QListView):
     @property
     def first_visible_row(self):
         geom = self.viewport().geometry()
-        for y in xrange(geom.top(), (self.spacing()*2) + geom.top(), 5):
-            for x in xrange(geom.left(), (self.spacing()*2) + geom.left(), 5):
+        for y in range(geom.top(), (self.spacing()*2) + geom.top(), 5):
+            for x in range(geom.left(), (self.spacing()*2) + geom.left(), 5):
                 ans = self.indexAt(QPoint(x, y)).row()
                 if ans > -1:
                     return ans
@@ -748,8 +758,8 @@ class GridView(QListView):
     @property
     def last_visible_row(self):
         geom = self.viewport().geometry()
-        for y in xrange(geom.bottom(), geom.bottom() - 2 * self.spacing(), -5):
-            for x in xrange(geom.left(), (self.spacing()*2) + geom.left(), 5):
+        for y in range(geom.bottom(), geom.bottom() - 2 * self.spacing(), -5):
+            for x in range(geom.left(), (self.spacing()*2) + geom.left(), 5):
                 ans = self.indexAt(QPoint(x, y)).row()
                 if ans > -1:
                     item_width = self.delegate.item_size.width() + 2*self.spacing()
@@ -759,7 +769,7 @@ class GridView(QListView):
         self.ignore_render_requests.clear()
         self.update_timer.stop()
         m = self.model()
-        for r in xrange(self.first_visible_row or 0, self.last_visible_row or (m.count() - 1)):
+        for r in range(self.first_visible_row or 0, self.last_visible_row or (m.count() - 1)):
             self.update(m.index(r, 0))
 
     def start_view_animation(self, index):
@@ -787,23 +797,27 @@ class GridView(QListView):
 
     def set_color(self):
         r, g, b = gprefs['cover_grid_color']
-        pal = QPalette()
-        col = QColor(r, g, b)
-        pal.setColor(pal.Base, col)
         tex = gprefs['cover_grid_texture']
+        pal = self.palette()
+        pal.setColor(pal.Base, QColor(r, g, b))
+        self.setPalette(pal)
+        ss = ''
         if tex:
             from calibre.gui2.preferences.texture_chooser import texture_path
             path = texture_path(tex)
             if path:
+                path = os.path.abspath(path).replace(os.sep, '/')
+                ss += 'background-image: url({});'.format(path)
+                ss += 'background-attachment: fixed;'
                 pm = QPixmap(path)
                 if not pm.isNull():
                     val = pm.scaled(1, 1).toImage().pixel(0, 0)
                     r, g, b = qRed(val), qGreen(val), qBlue(val)
-                    pal.setBrush(pal.Base, QBrush(pm))
-        dark = (r + g + b)/3.0 < 128
-        pal.setColor(pal.Text, QColor(Qt.white if dark else Qt.black))
-        self.setPalette(pal)
-        self.delegate.highlight_color = pal.color(pal.Text)
+        dark = max(r, g, b) < 115
+        col = '#eee' if dark else '#111'
+        ss += 'color: {};'.format(col)
+        self.delegate.highlight_color = QColor(col)
+        self.setStyleSheet('QListView {{ {} }}'.format(ss))
 
     def refresh_settings(self):
         size_changed = (
@@ -822,14 +836,17 @@ class GridView(QListView):
             self.delegate.calculate_spacing()
             self.setSpacing(self.delegate.spacing)
         self.set_color()
-        if size_changed:
-            dpr = self.device_pixel_ratio
-            self.thumbnail_cache.set_thumbnail_size(int(dpr * self.delegate.cover_size.width()), int(dpr*self.delegate.cover_size.height()))
+        self.set_thumbnail_cache_image_size()
         cs = gprefs['cover_grid_disk_cache_size']
         if (cs*(1024**2)) != self.thumbnail_cache.max_size:
             self.thumbnail_cache.set_size(cs)
         self.update_memory_cover_cache_size()
 
+    def set_thumbnail_cache_image_size(self):
+        dpr = self.device_pixel_ratio
+        self.thumbnail_cache.set_thumbnail_size(
+            int(dpr * self.delegate.cover_size.width()), int(dpr*self.delegate.cover_size.height()))
+
     def resizeEvent(self, ev):
         self._ncols = None
         self.resize_timer.start()
@@ -874,6 +891,9 @@ class GridView(QListView):
     def render_cover(self, book_id):
         if self.ignore_render_requests.is_set():
             return
+        dpr = self.device_pixel_ratio
+        page_width = int(dpr * self.delegate.cover_size.width())
+        page_height = int(dpr * self.delegate.cover_size.height())
         tcdata, timestamp = self.thumbnail_cache[book_id]
         use_cache = False
         if timestamp is None:
@@ -889,7 +909,6 @@ class GridView(QListView):
         if has_cover:
             p = QImage()
             p.loadFromData(cdata, CACHE_FORMAT if cdata is tcdata else 'JPEG')
-            dpr = self.device_pixel_ratio
             p.setDevicePixelRatio(dpr)
             if p.isNull() and cdata is tcdata:
                 # Invalid image in cache
@@ -901,7 +920,7 @@ class GridView(QListView):
                 if cdata is not None:
                     width, height = p.width(), p.height()
                     scaled, nwidth, nheight = fit_image(
-                        width, height, int(dpr * self.delegate.cover_size.width()), int(dpr * self.delegate.cover_size.height()))
+                        width, height, page_width, page_height)
                     if scaled:
                         if self.ignore_render_requests.is_set():
                             return
@@ -956,7 +975,7 @@ class GridView(QListView):
                 # rendered, but this is better than a deadlock
                 join_with_timeout(self.delegate.render_queue)
             except RuntimeError:
-                print ('Cover rendering thread is stuck!')
+                print('Cover rendering thread is stuck!')
             finally:
                 self.ignore_render_requests.clear()
         else:
@@ -969,7 +988,7 @@ class GridView(QListView):
         # Create a range based selector for each set of contiguous rows
         # as supplying selectors for each individual row causes very poor
         # performance if a large number of rows has to be selected.
-        for k, g in itertools.groupby(enumerate(rows), lambda (i,x):i-x):
+        for k, g in itertools.groupby(enumerate(rows), lambda i_x:i_x[0]-i_x[1]):
             group = list(map(operator.itemgetter(1), g))
             sel.merge(QItemSelection(m.index(min(group), 0), m.index(max(group), 0)), sm.Select)
         sm.select(sel, sm.ClearAndSelect)
@@ -1041,12 +1060,15 @@ class GridView(QListView):
     def number_of_columns(self):
         # Number of columns currently visible in the grid
         if self._ncols is None:
+            dpr = self.device_pixel_ratio
+            width = int(dpr * self.delegate.cover_size.width())
+            height = int(dpr * self.delegate.cover_size.height())
             step = max(10, self.spacing())
-            for y in range(step, 500, step):
-                for x in range(step, 500, step):
+            for y in range(step, 2 * height, step):
+                for x in range(step, 2 * width, step):
                     i = self.indexAt(QPoint(x, y))
                     if i.isValid():
-                        for x in range(self.viewport().width() - step, self.viewport().width() - 300, -step):
+                        for x in range(self.viewport().width() - step, self.viewport().width() - width, -step):
                             j = self.indexAt(QPoint(x, y))
                             if j.isValid():
                                 self._ncols = j.row() - i.row() + 1
@@ -1062,7 +1084,8 @@ class GridView(QListView):
             if not ci.isValid():
                 return
             c = ci.row()
-            delta = {Qt.Key_Left: -1, Qt.Key_Right: 1, Qt.Key_Up: -self.number_of_columns(), Qt.Key_Down: self.number_of_columns()}[k]
+            ncols = self.number_of_columns() or 1
+            delta = {Qt.Key_Left: -1, Qt.Key_Right: 1, Qt.Key_Up: -ncols, Qt.Key_Down: ncols}[k]
             n = max(0, min(c + delta, self.model().rowCount(None) - 1))
             if n == c:
                 return
@@ -1128,7 +1151,7 @@ class GridView(QListView):
         return super(GridView, self).selectionCommand(index, event)
 
     def wheelEvent(self, ev):
-        if ev.phase() not in (Qt.ScrollUpdate, 0):
+        if ev.phase() not in (Qt.ScrollUpdate, 0, Qt.ScrollMomentum):
             return
         number_of_pixels = ev.pixelDelta()
         number_of_degrees = ev.angleDelta() / 8.0
@@ -1143,4 +1166,14 @@ class GridView(QListView):
         if abs(dy) > 0:
             b.setValue(b.value() - dy)
 
+    def paintEvent(self, ev):
+        dpr = self.device_pixel_ratio
+        page_width = int(dpr * self.delegate.cover_size.width())
+        page_height = int(dpr * self.delegate.cover_size.height())
+        size_changed = self.thumbnail_cache.set_thumbnail_size(page_width, page_height)
+        if size_changed:
+            self.delegate.cover_cache.clear()
+
+        return super(GridView, self).paintEvent(ev)
+
 # }}}
