@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2011, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -13,17 +12,15 @@ DEBUG_DIALOG = False
 import os, time
 from threading import Thread, Event
 from operator import attrgetter
-from Queue import Queue, Empty
 from io import BytesIO
 
 from PyQt5.Qt import (
     QStyledItemDelegate, QTextDocument, QRectF, QIcon, Qt, QApplication,
     QDialog, QVBoxLayout, QLabel, QDialogButtonBox, QStyle, QStackedWidget,
-    QWidget, QTableView, QGridLayout, QFontInfo, QPalette, QTimer, pyqtSignal,
+    QWidget, QTableView, QGridLayout, QPalette, QTimer, pyqtSignal,
     QAbstractTableModel, QSize, QListView, QPixmap, QModelIndex,
     QAbstractListModel, QRect, QTextBrowser, QStringListModel, QMenu,
-    QCursor, QHBoxLayout, QPushButton, QSizePolicy)
-from PyQt5.QtWebKitWidgets import QWebView
+    QCursor, QHBoxLayout, QPushButton, QSizePolicy, QSplitter)
 
 from calibre.customize.ui import metadata_plugins
 from calibre.ebooks.metadata import authors_to_string, rating_to_stars
@@ -33,13 +30,15 @@ from calibre.ebooks.metadata.book.base import Metadata
 from calibre.ebooks.metadata.opf2 import OPF
 from calibre.gui2 import error_dialog, rating_font, gprefs
 from calibre.gui2.progress_indicator import draw_snake_spinner
+from calibre.gui2.widgets2 import HTMLDisplay
 from calibre.utils.date import (utcnow, fromordinal, format_date,
         UNDEFINED_DATE, as_utc)
 from calibre.library.comments import comments_to_html
 from calibre import force_unicode
-from calibre.utils.config import tweaks
 from calibre.utils.ipc.simple_worker import fork_job, WorkerError
 from calibre.ptempfile import TemporaryDirectory
+from polyglot.builtins import iteritems, itervalues, unicode_type, range, getcwd
+from polyglot.queue import Queue, Empty
 # }}}
 
 
@@ -153,7 +152,7 @@ class ResultsModel(QAbstractTableModel):  # {{{
 
     def data_as_text(self, book, col):
         if col == 0:
-            return unicode(book.gui_rank+1)
+            return unicode_type(book.gui_rank+1)
         if col == 1:
             t = book.title if book.title else _('Unknown')
             a = authors_to_string(book.authors) if book.authors else ''
@@ -183,7 +182,7 @@ class ResultsModel(QAbstractTableModel):  # {{{
             return book
         elif role == Qt.ToolTipRole and col == 3:
             return (
-                _('The has cover indication is not fully\n'
+                _('The "has cover" indication is not fully\n'
                     'reliable. Sometimes results marked as not\n'
                     'having a cover will find a cover in the download\n'
                     'cover stage, and vice versa.'))
@@ -313,53 +312,54 @@ class ResultsView(QTableView):  # {{{
 # }}}
 
 
-class Comments(QWebView):  # {{{
+class Comments(HTMLDisplay):  # {{{
 
     def __init__(self, parent=None):
-        QWebView.__init__(self, parent)
+        HTMLDisplay.__init__(self, parent)
         self.setAcceptDrops(False)
-        self.setMaximumWidth(300)
-        self.setMinimumWidth(300)
+        self.wait_timer = QTimer(self)
+        self.wait_timer.timeout.connect(self.update_wait)
+        self.wait_timer.setInterval(800)
+        self.dots_count = 0
+        self.anchor_clicked.connect(self.link_activated)
 
-        palette = self.palette()
-        palette.setBrush(QPalette.Base, Qt.transparent)
-        self.page().setPalette(palette)
-        self.setAttribute(Qt.WA_OpaquePaintEvent, False)
-
-        self.page().setLinkDelegationPolicy(self.page().DelegateAllLinks)
-        self.linkClicked.connect(self.link_clicked)
-
-    def link_clicked(self, url):
+    def link_activated(self, url):
         from calibre.gui2 import open_url
         if url.scheme() in {'http', 'https'}:
             open_url(url)
 
-    def turnoff_scrollbar(self, *args):
-        self.page().mainFrame().setScrollBarPolicy(Qt.Horizontal, Qt.ScrollBarAlwaysOff)
+    def show_wait(self):
+        self.dots_count = 0
+        self.wait_timer.start()
+        self.update_wait()
+
+    def update_wait(self):
+        self.dots_count += 1
+        self.dots_count %= 10
+        self.dots_count = self.dots_count or 1
+        self.setHtml(
+            '<h2>'+_('Please wait')+
+            '<br><span id="dots">{}</span></h2>'.format('.' * self.dots_count))
 
     def show_data(self, html):
+        self.wait_timer.stop()
+
         def color_to_string(col):
             ans = '#000000'
             if col.isValid():
                 col = col.toRgb()
                 if col.isValid():
-                    ans = unicode(col.name())
+                    ans = unicode_type(col.name())
             return ans
 
-        fi = QFontInfo(QApplication.font(self.parent()))
-        f = fi.pixelSize()+1+int(tweaks['change_book_details_font_size_by'])
-        fam = unicode(fi.family()).strip().replace('"', '')
-        if not fam:
-            fam = 'sans-serif'
-
         c = color_to_string(QApplication.palette().color(QPalette.Normal,
                         QPalette.WindowText))
         templ = '''\
         <html>
             <head>
             <style type="text/css">
-                body, td {background-color: transparent; font-family: "%s"; font-size: %dpx; color: %s }
-                a { text-decoration: none; color: blue }
+                body, td {background-color: transparent; color: %s }
+                a { text-decoration: none; }
                 div.description { margin-top: 0; padding-top: 0; text-indent: 0 }
                 table { margin-bottom: 0; padding-bottom: 0; }
             </style>
@@ -370,15 +370,8 @@ class Comments(QWebView):  # {{{
             </div>
             </body>
         <html>
-        '''%(fam, f, c)
+        '''%(c,)
         self.setHtml(templ%html)
-
-    def sizeHint(self):
-        # This is needed, because on windows the dialog cannot be resized to
-        # so that this widgets height become < sizeHint().height(). Qt sets the
-        # sizeHint to (800, 600), which makes the dialog unusable on smaller
-        # screens.
-        return QSize(800, 300)
 # }}}
 
 
@@ -423,7 +416,7 @@ class IdentifyWorker(Thread):  # {{{
                         'single_identify', (self.title, self.authors,
                             self.identifiers), no_output=True, abort=self.abort)
                 self.results, covers, caches, log_dump = res['result']
-                self.results = [OPF(BytesIO(r), basedir=os.getcwdu(),
+                self.results = [OPF(BytesIO(r), basedir=getcwd(),
                     populate_spine=False).to_book_metadata() for r in self.results]
                 for r, cov in zip(self.results, covers):
                     r.has_cached_cover_url = cov
@@ -452,47 +445,41 @@ class IdentifyWidget(QWidget):  # {{{
         self.abort = Event()
         self.caches = {}
 
-        self.l = l = QGridLayout()
-        self.setLayout(l)
+        self.l = l = QVBoxLayout(self)
 
         names = ['<b>'+p.name+'</b>' for p in metadata_plugins(['identify']) if
                 p.is_configured()]
         self.top = QLabel('<p>'+_('calibre is downloading metadata from: ') +
             ', '.join(names))
         self.top.setWordWrap(True)
-        l.addWidget(self.top, 0, 0)
+        l.addWidget(self.top)
 
+        self.splitter = s = QSplitter(self)
+        s.setChildrenCollapsible(False)
+        l.addWidget(s, 100)
         self.results_view = ResultsView(self)
         self.results_view.book_selected.connect(self.emit_book_selected)
         self.get_result = self.results_view.get_result
-        l.addWidget(self.results_view, 1, 0)
+        s.addWidget(self.results_view)
 
         self.comments_view = Comments(self)
-        l.addWidget(self.comments_view, 1, 1)
+        s.addWidget(self.comments_view)
+        s.setStretchFactor(0, 2)
+        s.setStretchFactor(1, 1)
 
         self.results_view.show_details_signal.connect(self.comments_view.show_data)
 
         self.query = QLabel('download starting...')
         self.query.setWordWrap(True)
-        l.addWidget(self.query, 2, 0, 1, 2)
+        l.addWidget(self.query)
 
-        self.comments_view.show_data('<h2>'+_('Please wait')+
-                '<br><span id="dots">.</span></h2>'+
-                '''
-                <script type="text/javascript">
-                window.onload=function(){
-                    var dotspan = document.getElementById('dots');
-                    window.setInterval(function(){
-                        if(dotspan.textContent == '............'){
-                        dotspan.textContent = '.';
-                        }
-                        else{
-                        dotspan.textContent += '.';
-                        }
-                    }, 400);
-                }
-                </script>
-                ''')
+        self.comments_view.show_wait()
+        state = gprefs.get('metadata-download-identify-widget-splitter-state')
+        if state is not None:
+            s.restoreState(state)
+
+    def save_state(self):
+        gprefs['metadata-download-identify-widget-splitter-state'] = bytearray(self.splitter.saveState())
 
     def emit_book_selected(self, book):
         self.book_selected.emit(book, self.caches)
@@ -508,12 +495,12 @@ class IdentifyWidget(QWidget):  # {{{
             parts.append('authors:'+authors_to_string(authors))
             simple_desc += _('Authors: %s ') % authors_to_string(authors)
         if identifiers:
-            x = ', '.join('%s:%s'%(k, v) for k, v in identifiers.iteritems())
+            x = ', '.join('%s:%s'%(k, v) for k, v in iteritems(identifiers))
             parts.append(x)
             if 'isbn' in identifiers:
-                simple_desc += ' ISBN: %s' % identifiers['isbn']
+                simple_desc += 'ISBN: %s' % identifiers['isbn']
         self.query.setText(simple_desc)
-        self.log(unicode(self.query.text()))
+        self.log(unicode_type(self.query.text()))
 
         self.worker = IdentifyWorker(self.log, self.abort, title,
                 authors, identifiers, self.caches)
@@ -544,7 +531,7 @@ class IdentifyWidget(QWidget):  # {{{
                         'match your search. Try making the search <b>less '
                         'specific</b>. For example, use only the author\'s '
                         'last name and a single distinctive word from '
-                        'the title.<p>To see the full log, click Show Details.'),
+                        'the title.<p>To see the full log, click "Show details".'),
                     show=True, det_msg=log)
             self.rejected.emit()
             return
@@ -560,7 +547,7 @@ class IdentifyWidget(QWidget):  # {{{
 class CoverWorker(Thread):  # {{{
 
     def __init__(self, log, abort, title, authors, identifiers, caches):
-        Thread.__init__(self)
+        Thread.__init__(self, name='CoverWorker')
         self.daemon = True
 
         self.log, self.abort = log, abort
@@ -666,7 +653,7 @@ class CoversModel(QAbstractListModel):  # {{{
         text = (src + '\n' + sz)
         scaled = pmap.scaled(
             int(CoverDelegate.ICON_SIZE[0] * pmap.devicePixelRatio()), int(CoverDelegate.ICON_SIZE[1] * pmap.devicePixelRatio()),
-            Qt.IgnoreAspectRatio, Qt.SmoothTransformation)
+            Qt.KeepAspectRatio, Qt.SmoothTransformation)
         scaled.setDevicePixelRatio(pmap.devicePixelRatio())
         return (text, (scaled), pmap, waiting)
 
@@ -680,7 +667,7 @@ class CoversModel(QAbstractListModel):  # {{{
             return None
         if role == Qt.DecorationRole:
             return pmap
-        if role == Qt.DisplayRole:
+        if role == Qt.DisplayRole or role == Qt.ToolTipRole:
             return text
         if role == Qt.UserRole:
             return waiting
@@ -688,7 +675,7 @@ class CoversModel(QAbstractListModel):  # {{{
 
     def plugin_for_index(self, index):
         row = index.row() if hasattr(index, 'row') else index
-        for k, v in self.plugin_map.iteritems():
+        for k, v in iteritems(self.plugin_map):
             if row in v:
                 return k
 
@@ -703,7 +690,7 @@ class CoversModel(QAbstractListModel):  # {{{
                 return 1
             return pmap.width()*pmap.height()
         dcovers = sorted(self.covers[1:], key=keygen, reverse=True)
-        cmap = {i:self.plugin_for_index(i) for i in xrange(len(self.covers))}
+        cmap = {i:self.plugin_for_index(i) for i in range(len(self.covers))}
         for i, x in enumerate(self.covers[0:1] + dcovers):
             if not x[-1]:
                 good.append(x)
@@ -749,8 +736,8 @@ class CoversModel(QAbstractListModel):  # {{{
             if pmap.isNull():
                 return
             self.beginInsertRows(QModelIndex(), last_row, last_row)
-            for rows in self.plugin_map.itervalues():
-                for i in xrange(len(rows)):
+            for rows in itervalues(self.plugin_map):
+                for i in range(len(rows)):
                     if rows[i] >= last_row:
                         rows[i] += 1
             self.plugin_map[plugin].insert(-1, last_row)
@@ -759,7 +746,7 @@ class CoversModel(QAbstractListModel):  # {{{
         else:
             # single cover plugin
             idx = None
-            for plugin, rows in self.plugin_map.iteritems():
+            for plugin, rows in iteritems(self.plugin_map):
                 if plugin.name == plugin_name:
                     idx = rows[0]
                     break
@@ -800,13 +787,20 @@ class CoversView(QListView):  # {{{
 
         self.delegate = CoverDelegate(self)
         self.setItemDelegate(self.delegate)
-        self.delegate.needs_redraw.connect(self.viewport().update,
+        self.delegate.needs_redraw.connect(self.redraw_spinners,
                 type=Qt.QueuedConnection)
 
         self.doubleClicked.connect(self.chosen, type=Qt.QueuedConnection)
         self.setContextMenuPolicy(Qt.CustomContextMenu)
         self.customContextMenuRequested.connect(self.show_context_menu)
 
+    def redraw_spinners(self):
+        m = self.model()
+        for r in range(m.rowCount()):
+            idx = m.index(r)
+            if bool(m.data(idx, Qt.UserRole)):
+                m.dataChanged.emit(idx, idx)
+
     def select(self, num):
         current = self.model().index(num)
         sm = self.selectionModel()
@@ -816,6 +810,9 @@ class CoversView(QListView):  # {{{
         self.select(0)
         self.delegate.start_animation()
 
+    def stop(self):
+        self.delegate.stop_animation()
+
     def reset_covers(self):
         self.m.reset_covers()
 
@@ -830,7 +827,7 @@ class CoversView(QListView):  # {{{
     def show_context_menu(self, point):
         idx = self.currentIndex()
         if idx and idx.isValid() and not idx.data(Qt.UserRole):
-            m = QMenu()
+            m = QMenu(self)
             m.addAction(QIcon(I('view.png')), _('View this cover at full size'), self.show_cover)
             m.addAction(QIcon(I('edit-copy.png')), _('Copy this cover to clipboard'), self.copy_cover)
             m.exec_(QCursor.pos())
@@ -841,8 +838,8 @@ class CoversView(QListView):  # {{{
         if pmap is None and idx.row() == 0:
             pmap = self.model().cc
         if pmap is not None:
-            from calibre.gui2.viewer.image_popup import ImageView
-            d = ImageView(self, pmap, unicode(idx.data(Qt.DisplayRole) or ''), geom_name='metadata_download_cover_popup_geom')
+            from calibre.gui2.image_popup import ImageView
+            d = ImageView(self, pmap, unicode_type(idx.data(Qt.DisplayRole) or ''), geom_name='metadata_download_cover_popup_geom')
             d(use_exec=True)
 
     def copy_cover(self):
@@ -935,11 +932,16 @@ class CoversWidget(QWidget):  # {{{
         if num < 2:
             txt = _('Could not find any covers for <b>%s</b>')%self.book.title
         else:
-            txt = _('Found <b>%(num)d</b> possible covers for %(title)s. '
-                    'When the download completes, the covers will be sorted by size.')%dict(num=num-1,
-                            title=self.title)
+            if num == 2:
+                txt = _('Found a cover for {title}').format(title=self.title)
+            else:
+                txt = _(
+                    'Found <b>{num}</b> covers for {title}. When the download completes,'
+                    ' the covers will be sorted by size.').format(
+                            title=self.title, num=num-1)
         self.msg.setText(txt)
         self.msg.setWordWrap(True)
+        self.covers_view.stop()
 
         self.finished.emit()
 
@@ -1046,7 +1048,7 @@ class FullFetch(QDialog):  # {{{
         self.prev_button = pb = QPushButton(QIcon(I('back.png')), _('&Back'), self)
         pb.clicked.connect(self.back_clicked)
         pb.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
-        self.log_button = self.bb.addButton(_('View log'), self.bb.ActionRole)
+        self.log_button = self.bb.addButton(_('&View log'), self.bb.ActionRole)
         self.log_button.clicked.connect(self.view_log)
         self.log_button.setIcon(QIcon(I('debug.png')))
         self.prev_button.setVisible(False)
@@ -1065,7 +1067,7 @@ class FullFetch(QDialog):  # {{{
         self.resize(850, 600)
         geom = gprefs.get('metadata_single_gui_geom', None)
         if geom is not None and geom:
-            self.restoreGeometry(geom)
+            QApplication.instance().safe_restore_geometry(self, geom)
 
         self.finished.connect(self.cleanup)
 
@@ -1090,6 +1092,7 @@ class FullFetch(QDialog):  # {{{
     def accept(self):
         # Prevent the usual dialog accept mechanisms from working
         gprefs['metadata_single_gui_geom'] = bytearray(self.saveGeometry())
+        self.identify_widget.save_state()
         if DEBUG_DIALOG:
             if self.stack.currentIndex() == 2:
                 return QDialog.accept(self)
@@ -1159,7 +1162,7 @@ class CoverFetch(QDialog):  # {{{
 
         self.bb = QDialogButtonBox(QDialogButtonBox.Cancel|QDialogButtonBox.Ok)
         l.addWidget(self.bb)
-        self.log_button = self.bb.addButton(_('View log'), self.bb.ActionRole)
+        self.log_button = self.bb.addButton(_('&View log'), self.bb.ActionRole)
         self.log_button.clicked.connect(self.view_log)
         self.log_button.setIcon(QIcon(I('debug.png')))
         self.bb.rejected.connect(self.reject)
@@ -1167,7 +1170,7 @@ class CoverFetch(QDialog):  # {{{
 
         geom = gprefs.get('single-cover-fetch-dialog-geometry', None)
         if geom is not None:
-            self.restoreGeometry(geom)
+            QApplication.instance().safe_restore_geometry(self, geom)
 
     def cleanup(self):
         self.covers_widget.cleanup()
@@ -1194,9 +1197,10 @@ class CoverFetch(QDialog):  # {{{
 
 # }}}
 
-if __name__ == '__main__':
-    DEBUG_DIALOG = True
-    app = QApplication([])
-    d = FullFetch()
-    d.start(title='great gatsby', authors=['fitzgerald'])
 
+if __name__ == '__main__':
+    from calibre.gui2 import Application
+    DEBUG_DIALOG = True
+    app = Application([])
+    d = FullFetch()
+    d.start(title='great gatsby', authors=['fitzgerald'], identifiers={})
