@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2012, Kovid Goyal <kovid@kovidgoyal.net>'
@@ -10,12 +9,13 @@ __docformat__ = 'restructuredtext en'
 from collections import namedtuple
 
 from PyQt5.Qt import (
-    QWidget, Qt, QLabel, QVBoxLayout, QPixmap, QDialogButtonBox, QApplication, QTimer,
+    QWidget, Qt, QLabel, QVBoxLayout, QDialogButtonBox, QApplication, QTimer, QPixmap,
     QSize, pyqtSignal, QIcon, QPlainTextEdit, QCheckBox, QPainter, QHBoxLayout, QFontMetrics,
-    QPainterPath, QRectF, pyqtProperty, QPropertyAnimation, QEasingCurve, QSizePolicy)
+    QPainterPath, QRectF, pyqtProperty, QPropertyAnimation, QEasingCurve, QSizePolicy, QImage)
 
 from calibre.constants import __version__
 from calibre.gui2.dialogs.message_box import ViewLog
+from polyglot.builtins import unicode_type
 
 Question = namedtuple('Question', 'payload callback cancel_callback '
         'title msg html_log log_viewer_title log_is_file det_msg '
@@ -23,6 +23,7 @@ Question = namedtuple('Question', 'payload callback cancel_callback '
         'action_label action_icon focus_action show_det show_ok icon '
         'log_viewer_unique_name')
 
+
 class Icon(QWidget):
 
     @pyqtProperty(float)
@@ -58,7 +59,7 @@ class Icon(QWidget):
         elif icon is None:
             self.icon = self.default_icon
         else:
-            self.icon = QPixmap(I(icon)).scaled(self.sizeHint(), transformMode=Qt.SmoothTransformation)
+            self.icon = QIcon(I(icon)).pixmap(self.sizeHint())
         self.update()
 
     def sizeHint(self):
@@ -70,6 +71,7 @@ class Icon(QWidget):
         p.drawPixmap(self.rect(), self.icon)
         p.end()
 
+
 class PlainTextEdit(QPlainTextEdit):
 
     def sizeHint(self):
@@ -78,6 +80,7 @@ class PlainTextEdit(QPlainTextEdit):
         ans.setWidth(fm.averageCharWidth() * 50)
         return ans
 
+
 class ProceedQuestion(QWidget):
 
     ask_question = pyqtSignal(object, object, object)
@@ -167,9 +170,9 @@ class ProceedQuestion(QWidget):
     def copy_to_clipboard(self, *args):
         QApplication.clipboard().setText(
                 'calibre, version %s\n%s: %s\n\n%s' %
-                (__version__, unicode(self.windowTitle()),
-                    unicode(self.msg_label.text()),
-                    unicode(self.det_msg.toPlainText())))
+                (__version__, unicode_type(self.windowTitle()),
+                    unicode_type(self.msg_label.text()),
+                    unicode_type(self.det_msg.toPlainText())))
         self.copy_button.setText(_('Copied'))
 
     def action_clicked(self):
@@ -207,7 +210,7 @@ class ProceedQuestion(QWidget):
         self.show_question()
 
     def toggle_det_msg(self, *args):
-        vis = unicode(self.det_msg_toggle.text()) == self.hide_det_msg
+        vis = unicode_type(self.det_msg_toggle.text()) == self.hide_det_msg
         self.det_msg_toggle.setText(self.show_det_msg if vis else
                 self.hide_det_msg)
         self.det_msg.setVisible(not vis)
@@ -263,9 +266,18 @@ class ProceedQuestion(QWidget):
         if self.rendered_pixmap is not None:
             return
 
-        p = QPixmap(self.size())
-        self.render(p)
-        self.rendered_pixmap = p
+        dpr = getattr(self, 'devicePixelRatioF', self.devicePixelRatio)()
+        p = QImage(dpr * self.size(), QImage.Format_ARGB32_Premultiplied)
+        p.setDevicePixelRatio(dpr)
+        # For some reason, Qt scrolls the book view when rendering this widget,
+        # for the very first time, so manually preserve its position
+        pr = getattr(self.parent(), 'library_view', None)
+        if not hasattr(pr, 'preserve_state'):
+            self.render(p)
+        else:
+            with pr.preserve_state():
+                self.render(p)
+        self.rendered_pixmap = QPixmap.fromImage(p)
         self.original_visibility = v = []
         for child in self.findChildren(QWidget):
             if child.isVisible():
@@ -390,6 +402,7 @@ class ProceedQuestion(QWidget):
         p.addRoundedRect(QRectF(self.rect()).adjusted(bw, bw, -bw, -bw), br, br)
         painter.fillPath(p, pal.color(pal.WindowText))
 
+
 def main():
     from calibre.gui2 import Application
     from PyQt5.Qt import QMainWindow, QStatusBar, QTimer
@@ -400,6 +413,7 @@ def main():
     s.showMessage('Testing ProceedQuestion')
     w.show()
     p = ProceedQuestion(w)
+
     def doit():
         p.dummy_question()
         p.dummy_question(action_label='A very long button for testing relayout (indeed)')
@@ -410,6 +424,6 @@ def main():
     QTimer.singleShot(10, doit)
     app.exec_()
 
+
 if __name__ == '__main__':
     main()
-
