@@ -9,6 +9,7 @@
 #include <QStyleOptionToolButton>
 #include <QFormLayout>
 #include <QDialogButtonBox>
+#include <QPainterPath>
 #include <algorithm>
 #include <qdrawutil.h>
 
@@ -173,6 +174,10 @@ class CalibreStyle: public QProxyStyle {
                     return QFormLayout::FieldsStayAtSizeHint;  // Do not have fields expand to fill all available space in QFormLayout
                 case SH_ScrollBar_Transient:
                     return transient_scroller;
+#ifdef Q_OS_MAC
+				case SH_UnderlineShortcut:
+					return 0;
+#endif
 				default:
                     break;
             }
@@ -180,6 +185,10 @@ class CalibreStyle: public QProxyStyle {
         }
 
         QIcon standardIcon(StandardPixmap standardIcon, const QStyleOption * option = 0, const QWidget * widget = 0) const {
+            if (standardIcon == QStyle::SP_DialogCloseButton) {
+                bool is_dark_theme = QApplication::instance()->property("is_dark_theme").toBool();
+                return QIcon(icon_map.value(QStyle::SP_CustomBase + (is_dark_theme ? 2 : 1)));
+            }
             if (icon_map.contains(standardIcon)) return QIcon(icon_map.value(standardIcon));
             return QProxyStyle::standardIcon(standardIcon, option, widget);
         }
@@ -188,6 +197,8 @@ class CalibreStyle: public QProxyStyle {
             switch (metric) {
                 case PM_TabBarTabVSpace:
                     return 8;  // Make tab bars a little narrower, the value for the Fusion style is 12
+				case PM_TreeViewIndentation:
+					return 10;  // Reduce indentation in tree views
                 default:
                     break;
             }
@@ -243,6 +254,39 @@ class CalibreStyle: public QProxyStyle {
 					}
 					break; // }}}
 
+				case PE_IndicatorRadioButton: // {{{
+					// Fix color used to draw radiobutton outline in dark mode
+					if (is_color_dark(option->palette.color(QPalette::Window))) {
+						painter->save();
+						painter->setBrush((option->state & State_Sunken) ? option->palette.base().color().lighter(320) : option->palette.base().color());
+						painter->setRenderHint(QPainter::Antialiasing, true);
+						QPainterPath circle;
+						const QPointF circleCenter = option->rect.center() + QPoint(1, 1);
+						const qreal outlineRadius = (option->rect.width() + (option->rect.width() + 1) % 2) / 2.0 - 1;
+						circle.addEllipse(circleCenter, outlineRadius, outlineRadius);
+						painter->setPen(QPen(option->palette.window().color().lighter(320)));
+						if (option->state & State_HasFocus && option->state & State_KeyboardFocusChange) {
+							QColor highlightedOutline = option->palette.color(QPalette::Highlight).lighter(125);
+							painter->setPen(QPen(highlightedOutline));
+						}
+						painter->drawPath(circle);
+
+						if (option->state & (State_On )) {
+							circle = QPainterPath();
+							const qreal checkmarkRadius = outlineRadius / 2.32;
+							circle.addEllipse(circleCenter, checkmarkRadius, checkmarkRadius);
+							QColor checkMarkColor = option->palette.text().color().lighter(120);
+							checkMarkColor.setAlpha(200);
+							painter->setPen(checkMarkColor);
+							checkMarkColor.setAlpha(180);
+							painter->setBrush(checkMarkColor);
+							painter->drawPath(circle);
+						}
+						painter->restore();
+						return;
+					} else { baseStyle()->drawPrimitive(element, option, painter, widget); }
+					break; // }}}
+
                 case PE_PanelItemViewItem:  // {{{
                     // Highlight the current, selected item with a different background in an item view if the highlight current item property is set
                     if (option->state & QStyle::State_HasFocus && (vopt = qstyleoption_cast<const QStyleOptionViewItem *>(option)) && widget && widget->property("highlight_current_item").toBool()) {
