@@ -15,15 +15,13 @@ from PyQt5.Qt import (
     pyqtSignal, QCompleter, QAction, QKeySequence, QTimer,
     QIcon, QMenu, QApplication, QKeyEvent)
 
-from calibre.gui2 import config, error_dialog, question_dialog, gprefs
+from calibre.gui2 import config, error_dialog, question_dialog, gprefs, QT_HIDDEN_CLEAR_ACTION
 from calibre.gui2.dialogs.confirm_delete import confirm
 from calibre.gui2.dialogs.saved_search_editor import SavedSearchEditor
 from calibre.gui2.dialogs.search import SearchDialog
 from calibre.utils.icu import primary_sort_key
 from polyglot.builtins import native_string_type, unicode_type, string_or_bytes, map, range
 
-QT_HIDDEN_CLEAR_ACTION = '_q_qlineeditclearaction'
-
 
 class AsYouType(unicode_type):
 
@@ -35,6 +33,7 @@ class AsYouType(unicode_type):
 
 class SearchLineEdit(QLineEdit):  # {{{
     key_pressed = pyqtSignal(object)
+    clear_history = pyqtSignal()
     select_on_mouse_press = None
 
     def keyPressEvent(self, event):
@@ -49,14 +48,18 @@ class SearchLineEdit(QLineEdit):  # {{{
         self.parent().normalize_state()
         menu = self.createStandardContextMenu()
         menu.setAttribute(Qt.WA_DeleteOnClose)
-        for action in menu.actions():
-            if action.text().startswith(_('&Paste') + '\t'):
-                break
         ac = menu.addAction(_('Paste and &search'))
         ac.setEnabled(bool(QApplication.clipboard().text()))
         ac.setIcon(QIcon(I('search.png')))
         ac.triggered.connect(self.paste_and_search)
-        menu.insertAction(action, ac)
+        for action in menu.actions():
+            if action.text().startswith(_('&Paste') + '\t'):
+                menu.insertAction(action, ac)
+                break
+        else:
+            menu.addAction(ac)
+        menu.addSeparator()
+        menu.addAction(_('&Clear search history')).triggered.connect(self.clear_history)
         menu.exec_(ev.globalPos())
 
     def paste_and_search(self):
@@ -110,6 +113,7 @@ class SearchBox2(QComboBox):  # {{{
         QComboBox.__init__(self, parent)
         self.line_edit = SearchLineEdit(self)
         self.setLineEdit(self.line_edit)
+        self.line_edit.clear_history.connect(self.clear_history)
         if add_clear_action:
             self.lineEdit().setClearButtonEnabled(True)
             ac = self.findChild(QAction, QT_HIDDEN_CLEAR_ACTION)
@@ -140,8 +144,8 @@ class SearchBox2(QComboBox):  # {{{
             icon = QIcon(I(icon))
         return self.lineEdit().addAction(icon, position)
 
-    def initialize(self, opt_name, colorize=False, help_text=_('Search')):
-        self.as_you_type = config['search_as_you_type']
+    def initialize(self, opt_name, colorize=False, help_text=_('Search'), as_you_type=None):
+        self.as_you_type = config['search_as_you_type'] if as_you_type is None else as_you_type
         self.opt_name = opt_name
         items = []
         for item in config[opt_name]:
@@ -152,6 +156,10 @@ class SearchBox2(QComboBox):  # {{{
         self.colorize = colorize
         self.clear()
 
+    def clear_search_history(self):
+        config[self.opt_name] = []
+        self.clear()
+
     def hide_completer_popup(self):
         try:
             self.lineEdit().completer().popup().setVisible(False)
@@ -558,7 +566,7 @@ class SavedSearchBoxMixin(object):  # {{{
         self.save_search_button.setToolTip('<p>' +
          _("Save current search under the name shown in the box. "
            "Press and hold for a pop-up options menu.") + '</p>')
-        self.save_search_button.setMenu(QMenu())
+        self.save_search_button.setMenu(QMenu(self.save_search_button))
         self.save_search_button.menu().addAction(
                             QIcon(I('plus.png')),
                             _('Create Saved search'),
@@ -567,7 +575,7 @@ class SavedSearchBoxMixin(object):  # {{{
             QIcon(I('trash.png')), _('Delete Saved search'), self.saved_search.delete_current_search)
         self.save_search_button.menu().addAction(
             QIcon(I('search.png')), _('Manage Saved searches'), partial(self.do_saved_search_edit, None))
-        self.add_saved_search_button.setMenu(QMenu())
+        self.add_saved_search_button.setMenu(QMenu(self.add_saved_search_button))
         self.add_saved_search_button.menu().aboutToShow.connect(self.populate_add_saved_search_menu)
 
     def populate_add_saved_search_menu(self):
