@@ -1,7 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=utf-8
-from __future__ import (unicode_literals, division, absolute_import,
-                        print_function)
+
 
 __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -15,6 +14,7 @@ _xml_types = {'application/oebps-page-map+xml', 'application/vnd.adobe-page-temp
             guess_type('a.'+x) for x in ('ncx', 'opf', 'svg', 'xpgt', 'xml')}
 _js_types = {'application/javascript', 'application/x-javascript'}
 
+
 def syntax_from_mime(name, mime):
     for syntax, types in (('html', OEB_DOCS), ('css', OEB_STYLES), ('xml', _xml_types)):
         if mime in types:
@@ -29,8 +29,10 @@ def syntax_from_mime(name, mime):
     if mime.endswith('+xml'):
         return 'xml'
 
+
 all_text_syntaxes = frozenset({'text', 'html', 'xml', 'css', 'javascript'})
 
+
 def editor_from_syntax(syntax, parent=None):
     if syntax in all_text_syntaxes:
         from calibre.gui2.tweak_book.editor.widget import Editor
@@ -46,12 +48,15 @@ SPELL_LOCALE_PROPERTY = SPELL_PROPERTY + 1
 LINK_PROPERTY = SPELL_LOCALE_PROPERTY + 1
 TAG_NAME_PROPERTY = LINK_PROPERTY + 1
 CSS_PROPERTY = TAG_NAME_PROPERTY + 1
+CLASS_ATTRIBUTE_PROPERTY = CSS_PROPERTY + 1
+
 
 def syntax_text_char_format(*args):
     ans = QTextCharFormat(*args)
     ans.setProperty(SYNTAX_PROPERTY, True)
     return ans
 
+
 class StoreLocale(object):
 
     __slots__ = ('enabled',)
@@ -64,4 +69,6 @@ class StoreLocale(object):
 
     def __exit__(self, *args):
         self.enabled = False
+
+
 store_locale = StoreLocale()
