@@ -5,19 +5,23 @@
 
 import copy
 import json
-from collections import OrderedDict, Counter
+import regex
+import time
+from collections import Counter, OrderedDict
 from functools import partial
-
 from PyQt5.Qt import (
-    QAbstractListModel, QApplication, QCheckBox, QComboBox, QFont, QFrame,
-    QGridLayout, QHBoxLayout, QIcon, QItemSelection, QLabel, QLineEdit, QListView,
-    QMenu, QMimeData, QModelIndex, QPushButton, QScrollArea, QSize, QSizePolicy,
-    QStackedLayout, QStyledItemDelegate, Qt, QTimer, QToolBar, QToolButton,
-    QVBoxLayout, QWidget, pyqtSignal, QAction, QKeySequence
+    QAbstractListModel, QAction, QApplication, QCheckBox, QComboBox, QFont, QFrame,
+    QGridLayout, QHBoxLayout, QIcon, QItemSelection, QKeySequence, QLabel, QLineEdit,
+    QListView, QMenu, QMimeData, QModelIndex, QPushButton, QScrollArea, QSize,
+    QSizePolicy, QStackedLayout, QStyledItemDelegate, Qt, QTimer, QToolBar,
+    QToolButton, QVBoxLayout, QWidget, pyqtSignal
 )
 
-import regex
 from calibre import prepare_string_for_xml
+from calibre.constants import iswindows
+from calibre.ebooks.conversion.search_replace import (
+    REGEX_FLAGS, compile_regular_expression
+)
 from calibre.gui2 import choose_files, choose_save_file, error_dialog, info_dialog
 from calibre.gui2.dialogs.confirm_delete import confirm
 from calibre.gui2.dialogs.message_box import MessageBox
@@ -33,9 +37,9 @@ from calibre.gui2.tweak_book.function_replace import (
 from calibre.gui2.tweak_book.widgets import BusyCursor
 from calibre.gui2.widgets2 import FlowLayout, HistoryComboBox
 from calibre.utils.icu import primary_contains
-from calibre.ebooks.conversion.search_replace import REGEX_FLAGS, compile_regular_expression
-from polyglot.builtins import iteritems, unicode_type, range, error_message, filter, map
-
+from polyglot.builtins import (
+    error_message, filter, iteritems, map, range, unicode_type
+)
 
 # The search panel {{{
 
@@ -97,6 +101,7 @@ class HistoryBox(HistoryComboBox):
         self.clear_msg = clear_msg
         self.ignore_snip_expansion = False
         self.lineEdit().setClearButtonEnabled(True)
+        self.set_uniform_item_sizes(False)
 
     def event(self, ev):
         if ev.type() in (ev.ShortcutOverride, ev.KeyPress) and ev.key() == KEY and ev.modifiers() & MODIFIER:
@@ -203,12 +208,15 @@ class ModeBox(QComboBox):
 
     def __init__(self, parent):
         QComboBox.__init__(self, parent)
-        self.addItems([_('Normal'), _('Regex'), _('Regex-function')])
+        self.addItems([_('Normal'), _('Fuzzy'), _('Regex'), _('Regex-function')])
         self.setToolTip('<style>dd {margin-bottom: 1.5ex}</style>' + _(
             '''Select how the search expression is interpreted
             <dl>
             <dt><b>Normal</b></dt>
-            <dd>The search expression is treated as normal text, calibre will look for the exact text.</dd>
+            <dd>The search expression is treated as normal text, calibre will look for the exact text</dd>
+            <dt><b>Fuzzy</b></dt>
+            <dd>The search expression is treated as "fuzzy" which means spaces will match any space character,
+            including tabs and line breaks. Plain quotes will match the typographical equivalents, etc.</dd>
             <dt><b>Regex</b></dt>
             <dd>The search expression is interpreted as a regular expression. See the User Manual for more help on using regular expressions.</dd>
             <dt><b>Regex-function</b></dt>
@@ -217,11 +225,11 @@ class ModeBox(QComboBox):
 
     @property
     def mode(self):
-        return ('normal', 'regex', 'function')[self.currentIndex()]
+        return ('normal', 'fuzzy', 'regex', 'function')[self.currentIndex()]
 
     @mode.setter
     def mode(self, val):
-        self.setCurrentIndex({'regex':1, 'function':2}.get(val, 0))
+        self.setCurrentIndex({'fuzzy': 1, 'regex':2, 'function':3}.get(val, 0))
 
 
 class SearchWidget(QWidget):
@@ -344,8 +352,8 @@ class SearchWidget(QWidget):
                 self.functions.setText('')
 
     def mode_changed(self, idx):
-        self.da.setVisible(idx > 0)
-        function_mode = idx == 2
+        self.da.setVisible(idx > 1)
+        function_mode = idx == 3
         self.rl.setVisible(not function_mode)
         self.rl2.setVisible(function_mode)
         self.replace_text.setVisible(not function_mode)
@@ -739,10 +747,12 @@ class EditSearch(QFrame):  # {{{
                 self.function.setText('')
 
     def mode_changed(self, idx):
-        self.dot_all.setVisible(idx > 0)
-        self.functions_container.setVisible(idx == 2)
-        self.la3.setVisible(idx < 2)
-        self.replace.setVisible(idx < 2)
+        mode = self.mode_box.mode
+        self.dot_all.setVisible(mode in ('regex', 'function'))
+        function_mode = mode == 'function'
+        self.functions_container.setVisible(function_mode)
+        self.la3.setVisible(not function_mode)
+        self.replace.setVisible(not function_mode)
 
     def show_search(self, search=None, search_index=-1, state=None):
         self.title.setText('<h2>' + (_('Add search') if search_index == -1 else _('Edit search')))
@@ -918,8 +928,8 @@ class SavedSearches(QWidget):
                 (_('&Find'), 'find', _('Run the search using the selected entries.') + mulmsg),
                 (_('&Replace'), 'replace', _('Run replace using the selected entries.') + mulmsg),
                 (_('Replace a&nd Find'), 'replace-find', _('Run replace and then find using the selected entries.') + mulmsg),
-                (_('Replace &all'), 'replace-all', _('Run Replace All for all selected entries in the order selected')),
-                (_('&Count all'), 'count', _('Run Count All for all selected entries')),
+                (_('Replace &all'), 'replace-all', _('Run Replace all for all selected entries in the order selected')),
+                (_('&Count all'), 'count', _('Run Count all for all selected entries')),
         ]:
             self.action_button_map[action] = b = pb(text, tooltip, action)
             v.addWidget(b)
@@ -1270,9 +1280,13 @@ class InvalidRegex(regex.error):
 
 def get_search_regex(state):
     raw = state['find']
-    is_regex = state['mode'] != 'normal'
+    is_regex = state['mode'] not in ('normal', 'fuzzy')
     if not is_regex:
-        raw = regex.escape(raw, special_only=True)
+        if state['mode'] == 'fuzzy':
+            from calibre.gui2.viewer.search import text_to_regex
+            raw = text_to_regex(raw)
+        else:
+            raw = regex.escape(raw, special_only=True)
     flags = REGEX_FLAGS
     if not state['case_sensitive']:
         flags |= regex.IGNORECASE
@@ -1290,7 +1304,7 @@ def get_search_regex(state):
 
 def get_search_function(state):
     ans = state['replace']
-    is_regex = state['mode'] != 'normal'
+    is_regex = state['mode'] not in ('normal', 'fuzzy')
     if not is_regex:
         # We dont want backslash escape sequences interpreted in normal mode
         return lambda m: ans
@@ -1513,8 +1527,16 @@ def run_search(
             if n in editors:
                 editors[n].replace_data(raw)
             else:
-                with current_container().open(n, 'wb') as f:
-                    f.write(raw.encode('utf-8'))
+                try:
+                    with current_container().open(n, 'wb') as f:
+                        f.write(raw.encode('utf-8'))
+                except PermissionError:
+                    if not iswindows:
+                        raise
+                    time.sleep(2)
+                    with current_container().open(n, 'wb') as f:
+                        f.write(raw.encode('utf-8'))
+
         QApplication.restoreOverrideCursor()
         count_message(replace, count, show_diff=replace, count_map=count_map)
         return count
