@@ -22,14 +22,13 @@ from calibre.ebooks.oeb.polish.cover import get_raster_cover_name
 from calibre.ebooks.oeb.polish.utils import lead_text, guess_type
 from calibre.gui2 import error_dialog, choose_files, choose_save_file, info_dialog, choose_images
 from calibre.gui2.tweak_book import tprefs, current_container
-from calibre.gui2.widgets2 import Dialog as BaseDialog, HistoryComboBox
+from calibre.gui2.widgets2 import Dialog as BaseDialog, HistoryComboBox, to_plain_text, PARAGRAPH_SEPARATOR
 from calibre.utils.icu import primary_sort_key, sort_key, primary_contains, numeric_sort_key
-from calibre.utils.matcher import get_char, Matcher
+from calibre.utils.matcher import get_char, Matcher, DEFAULT_LEVEL1, DEFAULT_LEVEL2, DEFAULT_LEVEL3
 from calibre.gui2.complete2 import EditWithComplete
 from polyglot.builtins import iteritems, unicode_type, zip, getcwd, filter as ignore_me
 
 ROOT = QModelIndex()
-PARAGRAPH_SEPARATOR = '\u2029'
 ignore_me
 
 
@@ -276,9 +275,13 @@ def make_highlighted_text(emph, text, positions):
     return text
 
 
+def emphasis_style():
+    pal = QApplication.instance().palette()
+    return 'color: {}; font-weight: bold'.format(pal.color(pal.Link).name())
+
+
 class Results(QWidget):
 
-    EMPH = "color:magenta; font-weight:bold"
     MARGIN = 4
 
     item_selected = pyqtSignal()
@@ -356,7 +359,7 @@ class Results(QWidget):
         self.update()
 
     def make_text(self, text, positions):
-        text = QStaticText(make_highlighted_text(self.EMPH, text, positions))
+        text = QStaticText(make_highlighted_text(emphasis_style(), text, positions))
         text.setTextOption(self.text_option)
         text.setTextFormat(Qt.RichText)
         return text
@@ -409,11 +412,12 @@ class Results(QWidget):
 
 class QuickOpen(Dialog):
 
-    def __init__(self, items, parent=None):
-        self.matcher = Matcher(items)
+    def __init__(self, items, parent=None, title=None, name='quick-open', level1=DEFAULT_LEVEL1, level2=DEFAULT_LEVEL2, level3=DEFAULT_LEVEL3, help_text=None):
+        self.matcher = Matcher(items, level1=level1, level2=level2, level3=level3)
         self.matches = ()
         self.selected_result = None
-        Dialog.__init__(self, _('Choose file to edit'), 'quick-open', parent=parent)
+        self.help_text = help_text or self.default_help_text()
+        Dialog.__init__(self, title or _('Choose file to edit'), name, parent=parent)
 
     def sizeHint(self):
         ans = Dialog.sizeHint(self)
@@ -421,25 +425,29 @@ class QuickOpen(Dialog):
         ans.setHeight(max(600, ans.height()))
         return ans
 
+    def default_help_text(self):
+        example = '<pre>{0}i{1}mages/{0}c{1}hapter1/{0}s{1}cene{0}3{1}.jpg</pre>'.format(
+            '<span style="%s">' % emphasis_style(), '</span>')
+        chars = '<pre style="%s">ics3</pre>' % emphasis_style()
+
+        return _('''<p>Quickly choose a file by typing in just a few characters from the file name into the field above.
+        For example, if want to choose the file:
+        {example}
+        Simply type in the characters:
+        {chars}
+        and press Enter.''').format(example=example, chars=chars)
+
     def setup_ui(self):
         self.l = l = QVBoxLayout(self)
         self.setLayout(l)
 
         self.text = t = QLineEdit(self)
         t.textEdited.connect(self.update_matches)
+        t.setClearButtonEnabled(True)
+        t.setPlaceholderText(_('Search'))
         l.addWidget(t, alignment=Qt.AlignTop)
 
-        example = '<pre>{0}i{1}mages/{0}c{1}hapter1/{0}s{1}cene{0}3{1}.jpg</pre>'.format(
-            '<span style="%s">' % Results.EMPH, '</span>')
-        chars = '<pre style="%s">ics3</pre>' % Results.EMPH
-
-        self.help_label = hl = QLabel(_(
-            '''<p>Quickly choose a file by typing in just a few characters from the file name into the field above.
-        For example, if want to choose the file:
-        {example}
-        Simply type in the characters:
-        {chars}
-        and press Enter.''').format(example=example, chars=chars))
+        self.help_label = hl = QLabel(self.help_text)
         hl.setContentsMargins(50, 50, 50, 50), hl.setAlignment(Qt.AlignTop | Qt.AlignHCenter)
         l.addWidget(hl)
         self.results = Results(self)
@@ -507,7 +515,7 @@ class NamesDelegate(QStyledItemDelegate):
             to.setWrapMode(to.NoWrap)
             to.setAlignment(Qt.AlignLeft | Qt.AlignVCenter)
             positions = sorted(set(positions) - {-1}, reverse=True)
-            text = '<body>%s</body>' % make_highlighted_text(Results.EMPH, text, positions)
+            text = '<body>%s</body>' % make_highlighted_text(emphasis_style(), text, positions)
             doc = QTextDocument()
             c = 'rgb(%d, %d, %d)'%c.getRgb()[:3]
             doc.setDefaultStyleSheet(' body { color: %s }'%c)
@@ -877,7 +885,7 @@ class InsertSemantics(Dialog):
             ' For example, you can specify that a particular location is the dedication or the preface'
             ' or the Table of Contents and so on.\n\nFirst choose the type of semantic information, then'
             ' choose a file and optionally a location within the file to point to.\n\nThe'
-            ' semantic information will be written in the <guide> section of the opf file.'))
+            ' semantic information will be written in the <guide> section of the OPF file.'))
         d.resize(d.sizeHint())
         d.exec_()
 
@@ -1199,17 +1207,7 @@ class PlainTextEdit(QPlainTextEdit):  # {{{
         self.syntax = None
 
     def toPlainText(self):
-        # QPlainTextEdit's toPlainText implementation replaces nbsp with normal
-        # space, so we re-implement it using QTextCursor, which does not do
-        # that
-        c = self.textCursor()
-        c.clearSelection()
-        c.movePosition(c.Start)
-        c.movePosition(c.End, c.KeepAnchor)
-        ans = c.selectedText().replace(PARAGRAPH_SEPARATOR, '\n')
-        # QTextCursor pads the return value of selectedText with null bytes if
-        # non BMP characters such as 0x1f431 are present.
-        return ans.rstrip('\0')
+        return to_plain_text(self)
 
     def selected_text_from_cursor(self, cursor):
         return unicodedata.normalize('NFC', unicode_type(cursor.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0'))
@@ -1232,8 +1230,8 @@ class PlainTextEdit(QPlainTextEdit):  # {{{
             return True
 
     def windows_ignore_altgr_shortcut(self, ev):
-        import win32api, win32con
-        s = win32api.GetAsyncKeyState(win32con.VK_RMENU) & 0xffff  # VK_RMENU == R_ALT
+        from calibre_extensions import winutil
+        s = winutil.get_async_key_state(winutil.VK_RMENU)  # VK_RMENU == R_ALT
         return s & 0x8000
 
     def event(self, ev):
