@@ -101,22 +101,16 @@ static PyMethodDef module_methods[] = {
     {NULL, NULL, 0, NULL}        /* Sentinel */
 };
 
+static int
+exec_module(PyObject *module) { return 0; }
 
-static struct PyModuleDef cocoa_module = {
-    /* m_base     */ PyModuleDef_HEAD_INIT,
-    /* m_name     */ "cocoa",
-    /* m_doc      */ "",
-    /* m_size     */ -1,
-    /* m_methods  */ module_methods,
-    /* m_slots    */ 0,
-    /* m_traverse */ 0,
-    /* m_clear    */ 0,
-    /* m_free     */ 0,
+static PyModuleDef_Slot slots[] = { {Py_mod_exec, exec_module}, {0, NULL} };
+
+static struct PyModuleDef module_def = {
+    .m_base     = PyModuleDef_HEAD_INIT,
+    .m_name     = "cocoa",
+    .m_methods  = module_methods,
+    .m_slots    = slots,
 };
-CALIBRE_MODINIT_FUNC PyInit_cocoa(void) {
-    PyObject *m = PyModule_Create(&cocoa_module);
-    if (m == NULL) {
-        return NULL;
-    }
-    return m;
-}
+
+CALIBRE_MODINIT_FUNC PyInit_cocoa(void) { return PyModuleDef_Init(&module_def); }
